/*-----------------------------------------------------------------------------------------
   This software is in the public domain, furnished "as is", without technical
   support, and with no warranty, express or implied, as to its usefulness for
   any purpose.

   This sketch based on:
   APM Mode Switch by Stefan Gofferje - http://stefan.gofferje.net/
   APM Mode Switch by Helle
   SPI examples by Nick Gammon - http://www.gammon.com.au/spi

   16 analog inputs to OpenTX via ppm: The SPI slave sketch

   This sketch acts as SPI slave and is triggered by the master. It receives from the master at 
   intervals a PPM signal with 8 channels, in which the current states of the analogue inputs 
   of the master are stored as servo signals.
   This sketch also reads its own 8 analog inputs and maps the values to servo signals from 
   1000 to 2000 microseconds. Then it combines the 16 servo signals into a single 
   PPM stream with a duration of 38.5 milliseconds and outputs this timer interrupt 
   controlled to PIN 9.

   Author: Dirk Wiedemann
   Date: 2018/04/13

   Version 0.1
*/

#include <SPI.h>
#include "SPI_anything.h"


// Basic config
#define numchan 16           // number of chanels in PPM stream
#define servo_default 1500 // default servo value (1500 = center)
#define PPM_FrLen 38500    // PPM frame length in microseconds
#define PPM_PulseLen 300   // PPM pulse length
#define PPM_Polarity 1     // PPM polarity -  0 = neg, 1 = pos
#define PPM_outPin 9       // PPM output pin


// Channel PWM values
int ppm[numchan];
volatile int ppm2[8];
int ArdPin[]={A0,A1,A2,A3,A4,A5,A6,A7};
int Analog;
int Kanal;

volatile bool haveData = false;

void setup(){
/*     
  Attention: using the serial console conflicts with the SPI and freeze this sketch
  after a short time. You should use the terminal only temporary for debugging.
*/
  //Serial.begin(115200);      // only for short time debugging

  pinMode(MISO, OUTPUT);  // send on master in, slave out

  SPCR |= _BV(SPE);        // turn on SPI in slave mode
   
  // All channels to center
  for(int i=0; i<numchan; i++){
    ppm[i]= servo_default;
  }

  pinMode(PPM_outPin, OUTPUT);
  digitalWrite(PPM_outPin, !PPM_Polarity);
  
  // Set up timer interrupt
  cli();                    // disable interrupts
  TCCR1A = 0; 
  TCCR1B = 0;
  OCR1A = 100;              // Output compare register (2MHz / 100 = 20000 Hz)
  TCCR1B |= (1 << WGM12);   // CTC gesetzt durch WGM12
  TCCR1B |= (1 << CS11);    // Prescaler 1:8 bei 16 MHz = 2 MHz
  TIMSK1 |= (1 << OCIE1A);  // 
  SPI.attachInterrupt();
  sei();
}


void loop() {    // Hauptroutine
  if (haveData) {
    for(int i=0; i<numchan; i++){    
      if(i<8) {
        Analog = analogRead(ArdPin[i]);
      } else {
        Analog = ppm2[i-8];
      }
    ppm[i] = map(Analog, 0, 1023, 1000, 2000);
    //Serial.print(ppm[i]);      // only for short time debugging
    //Serial.print("  ");        // only for short time debugging
    }
    //Serial.println();          // only for short time debugging
    haveData = false;
  }
}

ISR(TIMER1_COMPA_vect){
  static boolean state = true;
  
  TCNT1 = 0;
  
  if(state) {
    digitalWrite(PPM_outPin, PPM_Polarity);
    OCR1A = PPM_PulseLen * 2;
    state = false;
  }
  else{
    static byte channel;
    static unsigned int rest;
  
    digitalWrite(PPM_outPin, !PPM_Polarity);
    state = true;

    if(channel >= numchan){
      channel = 0;
      rest = rest + PPM_PulseLen;
      OCR1A = (PPM_FrLen - rest) * 2;
      rest = 0;
    }
    else{
      OCR1A = (ppm[channel] - PPM_PulseLen) * 2;
      rest = rest + ppm[channel];
      channel++;
    }     
  }
}

//SPI interrupt routine
ISR (SPI_STC_vect)
  {
    SPI_readAnything_ISR (ppm2);
    haveData = true;
  }   // end of interrupt routine

